# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'calculateprojectionseries_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.8.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_CalculateProjectionSeries_Dialog(object):
    def setupUi(self, CalculateProjectionSeries_Dialog):
        CalculateProjectionSeries_Dialog.setObjectName("CalculateProjectionSeries_Dialog")
        CalculateProjectionSeries_Dialog.resize(762, 382)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(CalculateProjectionSeries_Dialog.sizePolicy().hasHeightForWidth())
        CalculateProjectionSeries_Dialog.setSizePolicy(sizePolicy)
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout(CalculateProjectionSeries_Dialog)
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout()
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_3 = QtWidgets.QLabel(CalculateProjectionSeries_Dialog)
        self.label_3.setWordWrap(True)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_2.addWidget(self.label_3)
        self.lineEdit_angleFile = QtWidgets.QLineEdit(CalculateProjectionSeries_Dialog)
        self.lineEdit_angleFile.setMinimumSize(QtCore.QSize(500, 0))
        self.lineEdit_angleFile.setObjectName("lineEdit_angleFile")
        self.horizontalLayout_2.addWidget(self.lineEdit_angleFile)
        self.btn_selectAngleFile = QtWidgets.QPushButton(CalculateProjectionSeries_Dialog)
        self.btn_selectAngleFile.setObjectName("btn_selectAngleFile")
        self.horizontalLayout_2.addWidget(self.btn_selectAngleFile)
        self.verticalLayout_8.addLayout(self.horizontalLayout_2)
        self.line = QtWidgets.QFrame(CalculateProjectionSeries_Dialog)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout_8.addWidget(self.line)
        self.label_9 = QtWidgets.QLabel(CalculateProjectionSeries_Dialog)
        self.label_9.setObjectName("label_9")
        self.verticalLayout_8.addWidget(self.label_9)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.label_7 = QtWidgets.QLabel(CalculateProjectionSeries_Dialog)
        self.label_7.setObjectName("label_7")
        self.verticalLayout_5.addWidget(self.label_7)
        self.label_2 = QtWidgets.QLabel(CalculateProjectionSeries_Dialog)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_5.addWidget(self.label_2)
        self.label_8 = QtWidgets.QLabel(CalculateProjectionSeries_Dialog)
        self.label_8.setObjectName("label_8")
        self.verticalLayout_5.addWidget(self.label_8)
        self.horizontalLayout_4.addLayout(self.verticalLayout_5)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout()
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.lineEdit_phi = QtWidgets.QLineEdit(CalculateProjectionSeries_Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEdit_phi.sizePolicy().hasHeightForWidth())
        self.lineEdit_phi.setSizePolicy(sizePolicy)
        self.lineEdit_phi.setMaxLength(6)
        self.lineEdit_phi.setAlignment(QtCore.Qt.AlignCenter)
        self.lineEdit_phi.setObjectName("lineEdit_phi")
        self.verticalLayout_6.addWidget(self.lineEdit_phi)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.lineEdit_thetaStart = QtWidgets.QLineEdit(CalculateProjectionSeries_Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEdit_thetaStart.sizePolicy().hasHeightForWidth())
        self.lineEdit_thetaStart.setSizePolicy(sizePolicy)
        self.lineEdit_thetaStart.setMaxLength(6)
        self.lineEdit_thetaStart.setAlignment(QtCore.Qt.AlignCenter)
        self.lineEdit_thetaStart.setObjectName("lineEdit_thetaStart")
        self.verticalLayout.addWidget(self.lineEdit_thetaStart)
        self.verticalLayout_6.addLayout(self.verticalLayout)
        self.lineEdit_psi = QtWidgets.QLineEdit(CalculateProjectionSeries_Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEdit_psi.sizePolicy().hasHeightForWidth())
        self.lineEdit_psi.setSizePolicy(sizePolicy)
        self.lineEdit_psi.setMinimumSize(QtCore.QSize(10, 20))
        self.lineEdit_psi.setBaseSize(QtCore.QSize(2, 0))
        self.lineEdit_psi.setMaxLength(6)
        self.lineEdit_psi.setAlignment(QtCore.Qt.AlignCenter)
        self.lineEdit_psi.setObjectName("lineEdit_psi")
        self.verticalLayout_6.addWidget(self.lineEdit_psi)
        self.horizontalLayout_8.addLayout(self.verticalLayout_6)
        self.label_4 = QtWidgets.QLabel(CalculateProjectionSeries_Dialog)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_8.addWidget(self.label_4)
        self.horizontalLayout.addLayout(self.horizontalLayout_8)
        self.verticalLayout_9 = QtWidgets.QVBoxLayout()
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.verticalLayout_9.addItem(spacerItem)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.lineEdit_thetaStep = QtWidgets.QLineEdit(CalculateProjectionSeries_Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEdit_thetaStep.sizePolicy().hasHeightForWidth())
        self.lineEdit_thetaStep.setSizePolicy(sizePolicy)
        self.lineEdit_thetaStep.setMaxLength(6)
        self.lineEdit_thetaStep.setAlignment(QtCore.Qt.AlignCenter)
        self.lineEdit_thetaStep.setObjectName("lineEdit_thetaStep")
        self.horizontalLayout_5.addWidget(self.lineEdit_thetaStep)
        self.label_5 = QtWidgets.QLabel(CalculateProjectionSeries_Dialog)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_5.addWidget(self.label_5)
        self.verticalLayout_9.addLayout(self.horizontalLayout_5)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.verticalLayout_9.addItem(spacerItem1)
        self.horizontalLayout.addLayout(self.verticalLayout_9)
        self.verticalLayout_10 = QtWidgets.QVBoxLayout()
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.verticalLayout_10.addItem(spacerItem2)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.lineEdit_thetaStop = QtWidgets.QLineEdit(CalculateProjectionSeries_Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEdit_thetaStop.sizePolicy().hasHeightForWidth())
        self.lineEdit_thetaStop.setSizePolicy(sizePolicy)
        self.lineEdit_thetaStop.setMaxLength(6)
        self.lineEdit_thetaStop.setAlignment(QtCore.Qt.AlignCenter)
        self.lineEdit_thetaStop.setObjectName("lineEdit_thetaStop")
        self.horizontalLayout_7.addWidget(self.lineEdit_thetaStop)
        self.label_6 = QtWidgets.QLabel(CalculateProjectionSeries_Dialog)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_7.addWidget(self.label_6)
        self.verticalLayout_10.addLayout(self.horizontalLayout_7)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.verticalLayout_10.addItem(spacerItem3)
        self.horizontalLayout.addLayout(self.verticalLayout_10)
        self.horizontalLayout_4.addLayout(self.horizontalLayout)
        self.verticalLayout_8.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.checkBox_saveAngles = QtWidgets.QCheckBox(CalculateProjectionSeries_Dialog)
        self.checkBox_saveAngles.setObjectName("checkBox_saveAngles")
        self.horizontalLayout_11.addWidget(self.checkBox_saveAngles)
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_11.addItem(spacerItem4)
        self.verticalLayout_8.addLayout(self.horizontalLayout_11)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.label = QtWidgets.QLabel(CalculateProjectionSeries_Dialog)
        self.label.setWordWrap(True)
        self.label.setObjectName("label")
        self.horizontalLayout_6.addWidget(self.label)
        self.lineEdit_outputFilename = QtWidgets.QLineEdit(CalculateProjectionSeries_Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEdit_outputFilename.sizePolicy().hasHeightForWidth())
        self.lineEdit_outputFilename.setSizePolicy(sizePolicy)
        self.lineEdit_outputFilename.setObjectName("lineEdit_outputFilename")
        self.horizontalLayout_6.addWidget(self.lineEdit_outputFilename)
        self.verticalLayout_8.addLayout(self.horizontalLayout_6)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.verticalLayout_8.addLayout(self.horizontalLayout_3)
        self.label_10 = QtWidgets.QLabel(CalculateProjectionSeries_Dialog)
        self.label_10.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_10.setObjectName("label_10")
        self.verticalLayout_8.addWidget(self.label_10)
        self.line_2 = QtWidgets.QFrame(CalculateProjectionSeries_Dialog)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.verticalLayout_8.addWidget(self.line_2)
        self.buttonBox = QtWidgets.QDialogButtonBox(CalculateProjectionSeries_Dialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout_8.addWidget(self.buttonBox)
        self.verticalLayout_2.addLayout(self.verticalLayout_8)
        self.horizontalLayout_9.addLayout(self.verticalLayout_2)

        self.retranslateUi(CalculateProjectionSeries_Dialog)
        self.buttonBox.accepted.connect(CalculateProjectionSeries_Dialog.accept)
        self.buttonBox.rejected.connect(CalculateProjectionSeries_Dialog.reject)
        QtCore.QMetaObject.connectSlotsByName(CalculateProjectionSeries_Dialog)
        CalculateProjectionSeries_Dialog.setTabOrder(self.lineEdit_angleFile, self.btn_selectAngleFile)
        CalculateProjectionSeries_Dialog.setTabOrder(self.btn_selectAngleFile, self.lineEdit_phi)
        CalculateProjectionSeries_Dialog.setTabOrder(self.lineEdit_phi, self.lineEdit_thetaStart)
        CalculateProjectionSeries_Dialog.setTabOrder(self.lineEdit_thetaStart, self.lineEdit_psi)
        CalculateProjectionSeries_Dialog.setTabOrder(self.lineEdit_psi, self.lineEdit_outputFilename)

    def retranslateUi(self, CalculateProjectionSeries_Dialog):
        _translate = QtCore.QCoreApplication.translate
        CalculateProjectionSeries_Dialog.setWindowTitle(_translate("CalculateProjectionSeries_Dialog", "Dialog"))
        self.label_3.setText(_translate("CalculateProjectionSeries_Dialog", "Filename Containing Euler Angles"))
        self.btn_selectAngleFile.setText(_translate("CalculateProjectionSeries_Dialog", "Browse"))
        self.label_9.setText(_translate("CalculateProjectionSeries_Dialog", "Generate Euler Angles"))
        self.label_7.setText(_translate("CalculateProjectionSeries_Dialog", "Phi"))
        self.label_2.setText(_translate("CalculateProjectionSeries_Dialog", "Theta (Tilt)"))
        self.label_8.setText(_translate("CalculateProjectionSeries_Dialog", "Psi"))
        self.label_4.setText(_translate("CalculateProjectionSeries_Dialog", "Start"))
        self.label_5.setText(_translate("CalculateProjectionSeries_Dialog", "Step"))
        self.label_6.setText(_translate("CalculateProjectionSeries_Dialog", "Stop"))
        self.checkBox_saveAngles.setText(_translate("CalculateProjectionSeries_Dialog", "Save Angles"))
        self.label.setText(_translate("CalculateProjectionSeries_Dialog", "Projection Filename"))
        self.label_10.setText(_translate("CalculateProjectionSeries_Dialog", "*.mrc, *.npy, or *.mat"))

