# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ProjectionCalculator_MainWindow.ui'
#
# Created by: PyQt5 UI code generator 5.8.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_ProjectionCalculator(object):
    def setupUi(self, ProjectionCalculator):
        ProjectionCalculator.setObjectName("ProjectionCalculator")
        ProjectionCalculator.resize(819, 1000)
        self.centralwidget = QtWidgets.QWidget(ProjectionCalculator)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.gridLayout_2 = QtWidgets.QGridLayout()
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.btn_clearModel = QtWidgets.QPushButton(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btn_clearModel.sizePolicy().hasHeightForWidth())
        self.btn_clearModel.setSizePolicy(sizePolicy)
        self.btn_clearModel.setObjectName("btn_clearModel")
        self.horizontalLayout_4.addWidget(self.btn_clearModel)
        self.btn_go = QtWidgets.QPushButton(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btn_go.sizePolicy().hasHeightForWidth())
        self.btn_go.setSizePolicy(sizePolicy)
        palette = QtGui.QPalette()
        brush = QtGui.QBrush(QtGui.QColor(179, 179, 179))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Button, brush)
        brush = QtGui.QBrush(QtGui.QColor(179, 179, 179))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Button, brush)
        brush = QtGui.QBrush(QtGui.QColor(179, 179, 179))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Button, brush)
        self.btn_go.setPalette(palette)
        self.btn_go.setObjectName("btn_go")
        self.horizontalLayout_4.addWidget(self.btn_go)
        self.gridLayout_2.addLayout(self.horizontalLayout_4, 2, 0, 1, 4)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.verticalSlider_theta = QtWidgets.QSlider(self.centralwidget)
        self.verticalSlider_theta.setMaximum(3600)
        self.verticalSlider_theta.setSingleStep(10)
        self.verticalSlider_theta.setPageStep(100)
        self.verticalSlider_theta.setOrientation(QtCore.Qt.Vertical)
        self.verticalSlider_theta.setObjectName("verticalSlider_theta")
        self.horizontalLayout_6.addWidget(self.verticalSlider_theta)
        self.gridLayout.addLayout(self.horizontalLayout_6, 0, 1, 1, 1)
        self.label_6 = QtWidgets.QLabel(self.centralwidget)
        self.label_6.setAlignment(QtCore.Qt.AlignCenter)
        self.label_6.setObjectName("label_6")
        self.gridLayout.addWidget(self.label_6, 2, 0, 1, 1)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.verticalSlider_phi = QtWidgets.QSlider(self.centralwidget)
        self.verticalSlider_phi.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.verticalSlider_phi.setMaximum(3600)
        self.verticalSlider_phi.setSingleStep(1)
        self.verticalSlider_phi.setPageStep(100)
        self.verticalSlider_phi.setOrientation(QtCore.Qt.Vertical)
        self.verticalSlider_phi.setTickPosition(QtWidgets.QSlider.NoTicks)
        self.verticalSlider_phi.setTickInterval(1)
        self.verticalSlider_phi.setObjectName("verticalSlider_phi")
        self.horizontalLayout_5.addWidget(self.verticalSlider_phi)
        self.gridLayout.addLayout(self.horizontalLayout_5, 0, 0, 1, 1)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.verticalSlider_psi = QtWidgets.QSlider(self.centralwidget)
        self.verticalSlider_psi.setMaximum(360)
        self.verticalSlider_psi.setSingleStep(10)
        self.verticalSlider_psi.setPageStep(100)
        self.verticalSlider_psi.setOrientation(QtCore.Qt.Vertical)
        self.verticalSlider_psi.setObjectName("verticalSlider_psi")
        self.horizontalLayout_7.addWidget(self.verticalSlider_psi)
        self.gridLayout.addLayout(self.horizontalLayout_7, 0, 2, 1, 1)
        self.lineEdit_psi = QtWidgets.QLineEdit(self.centralwidget)
        self.lineEdit_psi.setAlignment(QtCore.Qt.AlignCenter)
        self.lineEdit_psi.setObjectName("lineEdit_psi")
        self.gridLayout.addWidget(self.lineEdit_psi, 1, 2, 1, 1)
        self.label_8 = QtWidgets.QLabel(self.centralwidget)
        self.label_8.setAlignment(QtCore.Qt.AlignCenter)
        self.label_8.setObjectName("label_8")
        self.gridLayout.addWidget(self.label_8, 2, 2, 1, 1)
        self.lineEdit_theta = QtWidgets.QLineEdit(self.centralwidget)
        self.lineEdit_theta.setAlignment(QtCore.Qt.AlignCenter)
        self.lineEdit_theta.setObjectName("lineEdit_theta")
        self.gridLayout.addWidget(self.lineEdit_theta, 1, 1, 1, 1)
        self.label_7 = QtWidgets.QLabel(self.centralwidget)
        self.label_7.setAlignment(QtCore.Qt.AlignCenter)
        self.label_7.setObjectName("label_7")
        self.gridLayout.addWidget(self.label_7, 2, 1, 1, 1)
        self.lineEdit_phi = QtWidgets.QLineEdit(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEdit_phi.sizePolicy().hasHeightForWidth())
        self.lineEdit_phi.setSizePolicy(sizePolicy)
        self.lineEdit_phi.setAlignment(QtCore.Qt.AlignCenter)
        self.lineEdit_phi.setObjectName("lineEdit_phi")
        self.gridLayout.addWidget(self.lineEdit_phi, 1, 0, 1, 1)
        self.gridLayout_2.addLayout(self.gridLayout, 0, 3, 1, 1)
        self.verticalLayout_figure = QtWidgets.QVBoxLayout()
        self.verticalLayout_figure.setObjectName("verticalLayout_figure")
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.verticalLayout_figure.addLayout(self.horizontalLayout_9)
        self.gridLayout_2.addLayout(self.verticalLayout_figure, 0, 1, 1, 2)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_2 = QtWidgets.QLabel(self.centralwidget)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout.addWidget(self.label_2)
        self.lineEdit_modelFile = QtWidgets.QLineEdit(self.centralwidget)
        self.lineEdit_modelFile.setObjectName("lineEdit_modelFile")
        self.horizontalLayout.addWidget(self.lineEdit_modelFile)
        self.btn_selectModel = QtWidgets.QPushButton(self.centralwidget)
        self.btn_selectModel.setObjectName("btn_selectModel")
        self.horizontalLayout.addWidget(self.btn_selectModel)
        self.gridLayout_2.addLayout(self.horizontalLayout, 1, 0, 1, 4)
        self.gridLayout_3.addLayout(self.gridLayout_2, 0, 0, 1, 1)
        ProjectionCalculator.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(ProjectionCalculator)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 819, 22))
        self.menubar.setObjectName("menubar")
        ProjectionCalculator.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(ProjectionCalculator)
        self.statusbar.setObjectName("statusbar")
        ProjectionCalculator.setStatusBar(self.statusbar)

        self.retranslateUi(ProjectionCalculator)
        QtCore.QMetaObject.connectSlotsByName(ProjectionCalculator)
        ProjectionCalculator.setTabOrder(self.lineEdit_phi, self.lineEdit_theta)
        ProjectionCalculator.setTabOrder(self.lineEdit_theta, self.lineEdit_psi)
        ProjectionCalculator.setTabOrder(self.lineEdit_psi, self.lineEdit_modelFile)
        ProjectionCalculator.setTabOrder(self.lineEdit_modelFile, self.btn_selectModel)

    def retranslateUi(self, ProjectionCalculator):
        _translate = QtCore.QCoreApplication.translate
        ProjectionCalculator.setWindowTitle(_translate("ProjectionCalculator", "Projection Calculator"))
        self.btn_clearModel.setText(_translate("ProjectionCalculator", "Clear Model"))
        self.btn_go.setText(_translate("ProjectionCalculator", "Calculate Projection Series From Model"))
        self.label_6.setText(_translate("ProjectionCalculator", "Phi"))
        self.label_8.setText(_translate("ProjectionCalculator", "Psi"))
        self.label_7.setText(_translate("ProjectionCalculator", "Theta"))
        self.label_2.setText(_translate("ProjectionCalculator", "Model"))
        self.btn_selectModel.setText(_translate("ProjectionCalculator", "Browse"))

