# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'VolumeSlicer_MainWindow.ui'
#
# Created by: PyQt5 UI code generator 5.8.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_VolumeSlicer(object):
    def setupUi(self, VolumeSlicer):
        VolumeSlicer.setObjectName("VolumeSlicer")
        VolumeSlicer.resize(1800, 800)
        self.centralwidget = QtWidgets.QWidget(VolumeSlicer)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.hz_lyt_figures = QtWidgets.QHBoxLayout()
        self.hz_lyt_figures.setObjectName("hz_lyt_figures")
        self.vt_lyt_fig1 = QtWidgets.QVBoxLayout()
        self.vt_lyt_fig1.setObjectName("vt_lyt_fig1")
        self.hz_lyt_figures.addLayout(self.vt_lyt_fig1)
        self.vt_lyt_fig2 = QtWidgets.QVBoxLayout()
        self.vt_lyt_fig2.setObjectName("vt_lyt_fig2")
        self.hz_lyt_figures.addLayout(self.vt_lyt_fig2)
        self.vt_lyt_fig3 = QtWidgets.QVBoxLayout()
        self.vt_lyt_fig3.setObjectName("vt_lyt_fig3")
        self.hz_lyt_figures.addLayout(self.vt_lyt_fig3)
        self.verticalLayout_2.addLayout(self.hz_lyt_figures)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.hz_lyt_sliders = QtWidgets.QHBoxLayout()
        self.hz_lyt_sliders.setObjectName("hz_lyt_sliders")
        self.sldr_fig1 = QtWidgets.QVBoxLayout()
        self.sldr_fig1.setObjectName("sldr_fig1")
        self.scrlbr_fig1 = QtWidgets.QScrollBar(self.centralwidget)
        self.scrlbr_fig1.setOrientation(QtCore.Qt.Horizontal)
        self.scrlbr_fig1.setInvertedAppearance(False)
        self.scrlbr_fig1.setObjectName("scrlbr_fig1")
        self.sldr_fig1.addWidget(self.scrlbr_fig1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem)
        self.label = QtWidgets.QLabel(self.centralwidget)
        self.label.setObjectName("label")
        self.horizontalLayout_6.addWidget(self.label)
        self.horizontalLayout.addLayout(self.horizontalLayout_6)
        self.lineEdit_scrlbr1 = QtWidgets.QLineEdit(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEdit_scrlbr1.sizePolicy().hasHeightForWidth())
        self.lineEdit_scrlbr1.setSizePolicy(sizePolicy)
        self.lineEdit_scrlbr1.setAlignment(QtCore.Qt.AlignCenter)
        self.lineEdit_scrlbr1.setObjectName("lineEdit_scrlbr1")
        self.horizontalLayout.addWidget(self.lineEdit_scrlbr1)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.sldr_fig1.addLayout(self.horizontalLayout)
        self.hz_lyt_sliders.addLayout(self.sldr_fig1)
        self.sldr_fig2 = QtWidgets.QVBoxLayout()
        self.sldr_fig2.setObjectName("sldr_fig2")
        self.scrlbr_fig2 = QtWidgets.QScrollBar(self.centralwidget)
        self.scrlbr_fig2.setOrientation(QtCore.Qt.Horizontal)
        self.scrlbr_fig2.setObjectName("scrlbr_fig2")
        self.sldr_fig2.addWidget(self.scrlbr_fig2)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem2)
        self.label_2 = QtWidgets.QLabel(self.centralwidget)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_8.addWidget(self.label_2)
        self.horizontalLayout_2.addLayout(self.horizontalLayout_8)
        self.lineEdit_scrlbr2 = QtWidgets.QLineEdit(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEdit_scrlbr2.sizePolicy().hasHeightForWidth())
        self.lineEdit_scrlbr2.setSizePolicy(sizePolicy)
        self.lineEdit_scrlbr2.setAlignment(QtCore.Qt.AlignCenter)
        self.lineEdit_scrlbr2.setObjectName("lineEdit_scrlbr2")
        self.horizontalLayout_2.addWidget(self.lineEdit_scrlbr2)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem3)
        self.sldr_fig2.addLayout(self.horizontalLayout_2)
        self.hz_lyt_sliders.addLayout(self.sldr_fig2)
        self.sldr_fig3 = QtWidgets.QVBoxLayout()
        self.sldr_fig3.setObjectName("sldr_fig3")
        self.scrlbr_fig3 = QtWidgets.QScrollBar(self.centralwidget)
        self.scrlbr_fig3.setOrientation(QtCore.Qt.Horizontal)
        self.scrlbr_fig3.setObjectName("scrlbr_fig3")
        self.sldr_fig3.addWidget(self.scrlbr_fig3)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_9.addItem(spacerItem4)
        self.label_3 = QtWidgets.QLabel(self.centralwidget)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_9.addWidget(self.label_3)
        self.horizontalLayout_3.addLayout(self.horizontalLayout_9)
        self.lineEdit_scrlbr3 = QtWidgets.QLineEdit(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEdit_scrlbr3.sizePolicy().hasHeightForWidth())
        self.lineEdit_scrlbr3.setSizePolicy(sizePolicy)
        self.lineEdit_scrlbr3.setAlignment(QtCore.Qt.AlignCenter)
        self.lineEdit_scrlbr3.setObjectName("lineEdit_scrlbr3")
        self.horizontalLayout_3.addWidget(self.lineEdit_scrlbr3)
        spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem5)
        self.sldr_fig3.addLayout(self.horizontalLayout_3)
        self.hz_lyt_sliders.addLayout(self.sldr_fig3)
        self.verticalLayout.addLayout(self.hz_lyt_sliders)
        self.hz_lyt_btns = QtWidgets.QHBoxLayout()
        self.hz_lyt_btns.setObjectName("hz_lyt_btns")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.checkBox_lockcmap = QtWidgets.QCheckBox(self.centralwidget)
        self.checkBox_lockcmap.setObjectName("checkBox_lockcmap")
        self.horizontalLayout_5.addWidget(self.checkBox_lockcmap)
        self.hz_lyt_btns.addLayout(self.horizontalLayout_5)
        self.verticalLayout.addLayout(self.hz_lyt_btns)
        self.verticalLayout_2.addLayout(self.verticalLayout)
        VolumeSlicer.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(VolumeSlicer)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1800, 22))
        self.menubar.setObjectName("menubar")
        VolumeSlicer.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(VolumeSlicer)
        self.statusbar.setObjectName("statusbar")
        VolumeSlicer.setStatusBar(self.statusbar)

        self.retranslateUi(VolumeSlicer)
        QtCore.QMetaObject.connectSlotsByName(VolumeSlicer)

    def retranslateUi(self, VolumeSlicer):
        _translate = QtCore.QCoreApplication.translate
        VolumeSlicer.setWindowTitle(_translate("VolumeSlicer", "Volume Slicer"))
        self.label.setText(_translate("VolumeSlicer", "X = "))
        self.label_2.setText(_translate("VolumeSlicer", "Y = "))
        self.label_3.setText(_translate("VolumeSlicer", "Z = "))
        self.checkBox_lockcmap.setText(_translate("VolumeSlicer", "Lock Colormap"))

